/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package b2jlibs.graphics;

import static b2jlibs.graphics.GraphicsElement.Type.COLOR;
import static b2jlibs.graphics.GraphicsElement.Type.LINE;
import static b2jlibs.graphics.GraphicsElement.Type.RECT;
import static b2jlibs.graphics.GraphicsElement.Type.TEXT;
import java.awt.Color;
import java.awt.Font;
import java.awt.font.TextAttribute;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;

/**
 *
 * @author David
 */
public class GraphicsPanel extends javax.swing.JPanel {
    final public Input inputHandler = new Input();
    final public LinkedList<GraphicsElement> graphicsElementListBuffer1;
    final public LinkedList<GraphicsElement> graphicsElementListBuffer2;
    boolean backBufferIs1;
    private Color drawColor;
    private boolean cls=true;
    private boolean flip=true;
    private Font currentFont = new Font("Monospaced", 0, 12) ;
    private ArrayList<Font> loadedFonts = new ArrayList<>();
    
    public int loadFont(String fontName, int size, boolean bold, boolean italic, boolean underlined) {
        Font newFont = new Font(fontName, (bold?Font.BOLD:0)+(italic?Font.ITALIC:0), size);
        if (underlined) {
            Map<TextAttribute, Integer> fontAttributes = new HashMap<>();
            fontAttributes.put(TextAttribute.UNDERLINE, TextAttribute.UNDERLINE_ON);
            newFont = newFont.deriveFont(fontAttributes);
        }
        loadedFonts.add(newFont);
        return loadedFonts.indexOf(newFont);
    }
    public void setFont(int nr) {
        try {
            currentFont = loadedFonts.get(nr);
        } catch (IndexOutOfBoundsException e) {}
    }
    
    public void addGraphicsElement(GraphicsElement e) {
        if (backBufferIs1) {
            if (graphicsElementListBuffer1!=null) {
                synchronized (graphicsElementListBuffer1) {
                    graphicsElementListBuffer1.add(e);
                }
            }
        } else {
            if (graphicsElementListBuffer2!=null) {
                synchronized (graphicsElementListBuffer2) {
                    graphicsElementListBuffer2.add(e);
                }
            }
        }
    }
    
    public void cls() {
        cls = true;
        /*if (backBufferIs1) {
            synchronized (graphicsElementListBuffer1) {
                graphicsElementListBuffer1.clear();
            }
        } else {
            synchronized (graphicsElementListBuffer2) {
                graphicsElementListBuffer2.clear();
            }
        }*/
    }
    public void flip(boolean fsync) {
        if (fsync) {
            flip=!flip;
            synchronized(this) {
                try {
                    this.wait(10);
                } catch (InterruptedException ex) {}
            }
        } else {
            flip=!flip;
        }
    }
    
    /**
     * Creates new form GraphicsPanel
     */
    public GraphicsPanel() {
        initComponents();
        graphicsElementListBuffer1 = new LinkedList<>();
        graphicsElementListBuffer2 = new LinkedList<>();
        drawColor = Color.WHITE;
        this.setBackground(Color.BLACK);
        this.setDoubleBuffered(true);
        this.setFocusable(true);
        this.requestFocus();
        this.addKeyListener(inputHandler);
        this.addMouseListener(inputHandler);
        
    }
    
    @Override
    public void paintComponent(java.awt.Graphics g) {
        g.setFont(currentFont);
        if (cls) {
            super.paintComponent(g);
            cls=false;
        }
        if (flip) {
            backBufferIs1=!backBufferIs1;
            flip = !flip;
        }
        g.setColor(drawColor);
        
        
        if (backBufferIs1) {
            synchronized (graphicsElementListBuffer2) {
                for (GraphicsElement e : graphicsElementListBuffer2) {
                    if (graphicsElementListBuffer2.size()==0) {
                        System.out.println("got");
                    }
                    switch (e.type) {
                        case LINE: g.drawLine(e.vals[0], e.vals[1], e.vals[2], e.vals[3]);break;
                        case RECT: g.drawRect(e.vals[0], e.vals[1], e.vals[2], e.vals[3]);break;
                        case FILLEDRECT: g.fillRect(e.vals[0], e.vals[1], e.vals[2], e.vals[3]);break;
                        case OVAL: g.drawOval(e.vals[0], e.vals[1], e.vals[2], e.vals[3]);break;
                        case FILLEDOVAL: g.fillOval(e.vals[0], e.vals[1], e.vals[2], e.vals[3]);break;
                        case TEXT: g.drawString(e.stringVals[0],e.vals[0], e.vals[1]+g.getFont().getSize());;break;
                        case COLOR: drawColor=new Color(e.vals[0]%256,e.vals[1]%256,e.vals[2]%256);g.setColor(drawColor);break;
                    }
                }
                graphicsElementListBuffer2.clear();
            }
        } else {
            synchronized (graphicsElementListBuffer1) {
                for (GraphicsElement e : graphicsElementListBuffer1) {
                    switch (e.type) {
                        case LINE: g.drawLine(e.vals[0], e.vals[1], e.vals[2], e.vals[3]);break;
                        case RECT: g.drawRect(e.vals[0], e.vals[1], e.vals[2], e.vals[3]);break;
                        case FILLEDRECT: g.fillRect(e.vals[0], e.vals[1], e.vals[2], e.vals[3]);break;
                        case OVAL: g.drawOval(e.vals[0], e.vals[1], e.vals[2], e.vals[3]);break;
                        case FILLEDOVAL: g.fillOval(e.vals[0], e.vals[1], e.vals[2], e.vals[3]);break;
                        case TEXT: g.drawString(e.stringVals[0],e.vals[0], e.vals[1]+g.getFont().getSize());break;
                        case COLOR: drawColor=new Color(e.vals[0]%256,e.vals[1]%256,e.vals[2]%256);g.setColor(drawColor);break;
                    }
                }
                graphicsElementListBuffer1.clear();
            }
        }
        synchronized (this) {
            this.notify();
        }
    }
    
    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 400, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 300, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
}
